﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core.Networks;
using HeuristicLab.Core.Views;
using HeuristicLab.Networks.Programmable;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Visualization;

namespace HeuristicLab.Networks.Views.NetworkVisualization.Views {
  public class AddNodesChartMode : ChartMode {
    protected TypeSelectorDialog typeSelectorDialog;

    public override Image Image { get { return VSImageLibrary.RadialChart; } }
    public override string ToolTip { get { return "Add Nodes"; } }

    public AddNodesChartMode(ChartControl control) : base(control) { }

    public override void HandleOnMouseMove(object sender, MouseEventArgs e) {
      base.HandleOnMouseMove(sender, e);

      if (!(chartControl.Tag is IUserDefinedNetwork)) return;

      try {
        chartControl.PictureBox.Refresh();

        var size = new Size(200, 100);
        using (var graphics = chartControl.PictureBox.CreateGraphics())
        using (var pen = new Pen(Color.Gray) { DashStyle = DashStyle.Dash }) {
          graphics.DrawRectangle(pen,
            (int)Math.Round(e.X - size.Width * 0.5),
            (int)Math.Round(e.Y - size.Height * 0.5),
            size.Width,
            size.Height);
        }
      } finally {
        chartControl.PictureBox.Update();
      }
    }

    public override void HandleOnMouseUp(object sender, MouseEventArgs e) {
      base.HandleOnMouseUp(sender, e);

      switch (e.Button) {
        case MouseButtons.Left:
          var network = chartControl.Tag as IUserDefinedNetwork;
          if (network == null) return;

          var node = CreateNode(network);
          var visualizableNode = node as IVisualizableNode;
          if (visualizableNode == null) return;

          var center = chartControl.Chart.TransformPixelToWorld(e.Location);
          var size = chartControl.Chart.TransformPixelToWorld(new Size(200, 100));

          visualizableNode.VisualProperties.X = center.X;
          visualizableNode.VisualProperties.Y = center.Y;
          visualizableNode.VisualProperties.Width = size.Width;
          visualizableNode.VisualProperties.Height = size.Height;

          network.Nodes.Add(node);
          break;
      }
    }

    #region Helpers
    protected virtual INode CreateNode(INetwork network) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Node";
        typeSelectorDialog.TypeSelector.Caption = "Available Nodes";
        typeSelectorDialog.TypeSelector.Configure(typeof(INode), false, true);
      }

      if (typeSelectorDialog.ShowDialog(chartControl) == DialogResult.OK) {
        try {
          var n = (INode)typeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
          n.Name = GetUniqueName(n.Name, network);
          return n;
        } catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(chartControl, ex);
        }
      }
      return null;
    }

    protected virtual string GetUniqueName(string originalName, INetwork network) {
      if (!network.Nodes.ContainsKey(originalName))
        return originalName;

      string name;
      int index = 0;
      do {
        index++;
        name = originalName + index;
      } while (network.Nodes.ContainsKey(name));
      return name;
    }
    #endregion
  }
}
